@@ -1,5 +1,6 @@
-{-# LANGUAGE DeriveFunctor, EmptyCase, GADTs, RankNTypes #-}
-{-# LANGUAGE ScopedTypeVariables, TupleSections #-}
+{-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE DeriveFunctor, EmptyCase, FlexibleInstances, GADTs, RankNTypes #-}
+{-# LANGUAGE MultiParamTypeClasses, ScopedTypeVariables, TupleSections #-}
 module Sketch where
 
 import Control.Arrow hiding (first, second)
@@ -9,6 +10,9 @@ import Control.Selective
 import Data.Bifunctor
 import Data.Bool
 import Data.Function
+import Data.Functor
+import Data.Functor.Identity
+import Data.Functor.Const
 import Data.Semigroup (Semigroup (..))
 import Data.Void
 
@@ -330,11 +334,7 @@ fromM :: SelectiveM f => f (Either a b) -> f (a -> b) -> f b
 fromM x f = either id (\(a, f) -> f a) <$> (fmap swapEither x |**| fmap Right f)
 
 toM :: Selective f => f (Either e a) -> f (Either e b) -> f (Either e (a, b))
-toM a b = select ((fmap Left . swapEither) <$> a) ((\e a -> fmap (a,) e) <$> b)
-
--- | Swap @Left@ and @Right@.
-swapEither :: Either a b -> Either b a
-swapEither = either Right Left
+toM = biselect
 
 -- Proof that if select = selectM, and <*> = ap, then <*> = apS.
 apSEqualsApply :: (Selective f, Monad f) => f (a -> b) -> f a -> f b
@@ -448,6 +448,30 @@ t7 x y z =
         ((\mbc cd -> maybe (Right Nothing) (\bc -> Left $ fmap ((cd . bc) .)) mbc) <$> y))
         ((&) <$> z)
 
+------------------------ McCarthy's Conditional combinator -------------------------
+-- See: http://www4.di.uminho.pt/~jno/ps/pdbc.pdf
+-- And also: https://themattchan.com/docs/algprog.pdf
+
+-- Guard function used in McCarthy's conditional
+-- | It provides information about the outcome of testing @p@ on some input @a@,
+-- encoded in terms of the coproduct injections without losing the input
+-- @a@ itself.
+grdS :: Applicative f => f (a -> Bool) -> f a -> f (Either a a)
+grdS f a = selector <$> applyF f (dup <$> a)
+  where
+    dup x = (x, x)
+    applyF fab faa = bimap <$> fab <*> pure id <*> faa
+    selector (b, x) = bool (Right x) (Left x) b
+
+-- | McCarthy's conditional, denoted p -> f,g is a well-known functional
+-- combinator, which suggests that, to reason about conditionals, one may
+-- seek help in the algebra of coproducts.
+--
+-- This combinator is very similar to the very nature of the 'select'
+-- operator and benefits from a series of properties and laws.
+condS :: Selective f => f (b -> Bool) -> f (b -> c) -> f (b -> c) -> f b -> f c
+condS p f g = (\r -> branch r f g) . grdS p
+
 ------------------------ Carter Schonwald's copatterns -------------------------
 -- See: https://github.com/cartazio/symmetric-monoidal/blob/15b209953b7d4a47651f615b02dbb0171de8af40/src/Control/Monoidal.hs#L93
 -- And also: https://twitter.com/andreymokhov/status/1102648479841701888
@@ -469,6 +493,28 @@ branchC x l r = choose x $ Choice $ \c -> case c of { CLeft -> l; CRight -> r }
 chooseS :: Selective f => f (Either a b) -> Choice (f (a -> c)) (f (b -> c)) -> f c
 chooseS x (Choice c) = branch x (c CLeft) (c CRight)
 
+------------------------------- ApplicativeError -------------------------------
+-- See https://twitter.com/LukaJacobowitz/status/1148756733243940864.
+
+class Applicative f => ApplicativeEither f e where
+    raise  :: e -> f a
+    handle :: f a -> f (e -> a) -> f a -- Note that the handler may fail too
+
+-- If the first computation succeeds with an @a@, this function just returns it.
+-- Otherwise, it attempts to handle the error @e@ by running the second
+-- computation. If the latter fails too, we return the very first error @e@,
+-- otherwise we handle the error with the obtained function @e -> a@ and return
+-- the resulting value @a@.
+handleS :: Selective f => f (Either e a) -> f (Either e (e -> a)) -> f (Either e a)
+handleS x y = select (second Right <$> x) (handlePure <$> y)
+  where
+    handlePure :: Either e (e -> a) -> e -> Either e a
+    handlePure (Left  _) e = Left e
+    handlePure (Right f) e = Right (f e)
+
+instance Selective f => ApplicativeEither (ComposeEither f e) e where
+    raise                                      = ComposeEither . pure . Left
+    handle (ComposeEither x) (ComposeEither y) = ComposeEither (handleS x y)
 ------------------------------- Free ArrowChoice -------------------------------
 
 -- A free 'ArrowChoice' built on top of base components @f i o@.
@@ -547,6 +593,80 @@ instance Monad Haxl where
         case rx of Done       x -> runHaxl (f x) -- dynamic dependency on runtime value 'x'
                    Blocked bx x -> return (Blocked bx (x >>= f))
 
+-- right' :: Choice p => p a b -> p (Either c a) (Either c b)
+-- right' :: ... => (a -> f b) -> Either x a -> f (Either x b)
+
+data P s t a b = P { match' :: s -> Either a t, build' :: b -> t }
+
+fromP :: P s t a b -> Prism s t a b
+fromP (P match build) f s = case match s of
+    Left  a -> build <$> f a
+    Right t -> pure t
+
+
+-- Choice p, Applicative f) => p a (f b) -> p s (f t)
+
+-- (a -> f b) -> (Either a x -> f (Either b x))
+
+type Lens      s t a b = forall f. Functor f     => (a -> f b) -> s -> f t
+type Traversal s t a b = forall f. Applicative f => (a -> f b) -> s -> f t
+type Prism     s t a b = forall f. Selective f   => (a -> f b) -> s -> f t
+
+_fst :: Lens (a, x) (b, x) a b
+_fst f (a, x) = f a <&> (,x)
+
+_snd :: Lens (x, a) (x, b) a b
+_snd f (x, a) = (x,) <$> f a
+
+_Left :: Prism (Either a x) (Either b x) a b
+_Left f = \case Left  a -> Left <$> f a
+                Right x -> pure (Right x)
+
+view :: Lens s t a b -> s -> a
+view lens s = getConst $ lens (\a -> Const a) s
+
+update :: Lens s t a b -> b -> s -> t
+update lens b s = runIdentity $ lens (\_a -> Identity b) s
+
+match :: Prism s t a b -> s -> Either a t
+match prism s = prism Left s
+
+-- (a -> f s b) -> s -> f s t
+build :: Prism s t a b -> b -> t
+build prism b = either absurd id $ prism (\_a -> Right b) undefined
+
+
+-- x <*? (y <*? z)
+
+-- 1      0     0
+-- 1      1     0
+-- 1      1     1
+
+-- (x <*? y) <*? z
+
+-- 1      0      0
+-- 1      1      0
+-- 1      0      1
+-- 1      1      1
+
+
+-- data Evaluation e a = Unknown e | Known a | Wrapped (Evaluation e a)
+--     deriving (Functor, Show)
+
+-- type E e a = Identity
+
+-- instance Semigroup e => Applicative (Evaluation e) where
+--     pure = Known
+--     Unknown e1 <*> Unknown e2 = Unknown (e1 <> e2)
+--     Unknown e1 <*> Known _    = Unknown e1
+--     Known _    <*> Unknown e2 = Unknown e2
+--     Known f    <*> Known a    = Known (f a)
+
+-- instance Semigroup e => Selective (Evaluation e) where
+--     select (Known (Right b)) _            = Known b
+--     select (Known (Left  a)) f            = ($a) <$> f
+--     select (Unknown e1     ) (Known   _ ) = Unknown e1
+--     select (Unknown e1     ) (Unknown e2) = Unknown (e1 <> e2)
 
 
 data C f g a where
@@ -561,3 +681,35 @@ instance (Applicative f, Applicative g) => Applicative (C f g) where
         \(x1, x2) -> Left (\(y1, y2) ->
             (either ($y1) id (k1 x1)) (either ($y2) id (k2 x2)))
 
+-- class Applicative f => SelectiveQ f where
+--     select :: Enumerable t => f (g a) -> (t ~> f :*: u) -> f (Sigma u)
+
+-- instance (Selective f, Selective g) => Selective (f :*: g) where
+
+-- instance Selective (Maybe :*: Maybe) where
+
+-- instance Selective Maybe where
+--     match sigma pi = sigma >>= \s -> pureSelect s pi
+
+
+
+-- data Value a b = Likely a | Unlikely b
+-- Some kind of "Sigma arrows"
+-- Sigma t -> (Match t (Sigma s)) -> Sigma s
+-- Sigma t -> (t ~> Sigma s) -> Sigma s
+
+-- branchU :: forall f a b c. SelectiveU f => f (Either a b) -> f (a -> c) -> f (b -> c) -> f c
+-- branchU x f g = selectU (toAB <$> x) h
+--   where
+--     -- toAB :: Either a b -> (AB a b x, x)
+--     toAB (Left  a) = (A, a)
+--     toAB (Right b) = (B, b)
+--     -- h :: AB a b x -> f (x -> c)
+--     h A = f
+--     h B = g
+
+-- applyS :: forall f a b. SelectiveS f => f a -> f (a -> b) -> f b
+-- applyS x f = sigmaSelect (Sigma Refl <$> x) h
+--   where
+--     h :: forall x. Refl a x -> f (x -> b)
+--     h Refl = f
